@@ -104,7 +104,6 @@ int er_handler (Display * c, XErrorEvent * e)
 extern Atom ATOM_WM_PROTOCOLS, ATOM_WM_DELETE_WINDOW, ATOM_WM_NAME, ATOM_WM_NORMAL_HINTS;
 char *argv_nought = "cooledit";
 Window main_window;
-int display_dnd_protocol_change_message = 0;
 
 /* {{{ signal handling */
 
@@ -158,6 +157,11 @@ static void set_signals (void)
     signal (SIGINT, SIG_DFL);
     signal (SIGTERM, SIG_DFL);
 #endif
+#ifdef SIGTSTP
+    signal(SIGTSTP, SIG_IGN);
+    signal(SIGTTIN, SIG_IGN);
+    signal(SIGTTOU, SIG_IGN);
+#endif				/* SIGTSTP */
 }
 
 /* }}} signal handling */
@@ -220,6 +224,7 @@ void get_next_hint_message (void)
 	gettext_noop("Do not email the author with generic questions, goto your local LUG instead"),
 	gettext_noop("See how to do regular expression substring replacements in the man page"),
 	gettext_noop("Use Shift-Up/Down in input widgets to get a history of inputs"),
+	gettext_noop("Setup your personal file list ~/.cedit/filelist so that Jump To File works"),
 	gettext_noop("Run frequently used apps from a hot key: see the Scripts menu"),
 	gettext_noop("Get your keypad to work by redefining keys in the Options menu"),
 	gettext_noop("Drag a file name from the 'File browser' in the Window menu to insert it"),
@@ -235,6 +240,7 @@ void get_next_hint_message (void)
 	gettext_noop("Turn off these messages from the Options menu: 'Hint time...' = 0"),
 	gettext_noop("Do not email the author with generic questions, goto your local LUG instead"),
 	gettext_noop("See how to do regular expression substring replacements in the man page"),
+	gettext_noop("You can drag and drop columns of text between windows with the Ctrl key"),
 	gettext_noop("See cooledit.1 for a list of the many extended editing/movement keys"),
 	gettext_noop("Compose international characters with Right-Control"),
 	gettext_noop("Set the LC_ALL and LANG environment variables - see the FAQ"),
@@ -259,7 +265,6 @@ void get_next_hint_message (void)
 	gettext_noop("Use Ctrl-Tab to complete the start of string, C-function or LaTeX-macro"),
 	gettext_noop("Turn off tool hints from the Options menu"),
 	gettext_noop("Use `Auto paragraph...' and `Word wrap...' to edit paragraphs like a WP"),
-	gettext_noop("See how to get a rotating shaded 3D `e' when recieving mail - read coolicon.1"),
 	gettext_noop("See how to do regular expression substring replacements in the man page"),
 	gettext_noop("Try your hand at adding new Cooledit features with Python"),
 	gettext_noop("Mark locations in files with the book mark commands in the Edit menu"),
@@ -275,6 +280,7 @@ void get_next_hint_message (void)
 	gettext_noop("Bookmark all matching strings with the `Bookmarks' switch in the `Search' dialog"),
 	gettext_noop("Create a printable postscript manual with   man -t cooledit > cooledit.ps"),
 	gettext_noop("Read the Python online tutorials: http://www.python.org/"),
+	gettext_noop("Setup ~/.cedit/filelist so that Ctrl-Tab completes in your file-browser"),
 	gettext_noop("Enable \"Smooth scrolling\" option if you have a fast graphics card"),
 	gettext_noop("Start internal XTerms with \"Terminal\" in the command menu"),
 	gettext_noop("Use the coolproject command to start projects"),
@@ -752,7 +758,7 @@ static int bookmarks_done_callback (CWidget * w, XEvent * x, CEvent * c)
     return 1;
 }
 
-void goto_file_dialog (char *heading, char *tool_hint, char *text)
+void goto_file_dialog (const char *heading, const char *tool_hint, const char *text)
 {
     int x, y;
     Window win;
@@ -1259,6 +1265,31 @@ void menu_browse_cmd (unsigned long ignored)
     (CIdent (catstrs ("_cfileBr", itoa (l), 0)))->position |= WINDOW_UNMOVEABLE;
 }
 
+void menu_jump_to_file (unsigned long ignored)
+{
+    long cursor, end_of_name;
+    char *s, *f;
+
+    s = edit_get_current_line_as_text (edit[current_edit]->editor, NULL, &cursor);
+    while (cursor > 0 && s[cursor - 1] && strchr (NICE_FILENAME_CHARS "/", (unsigned char) s[cursor - 1]))
+	cursor--;
+    for (end_of_name = cursor;
+	 s[end_of_name] && strchr (NICE_FILENAME_CHARS "/", (unsigned char) s[end_of_name]); end_of_name++);
+    s[end_of_name] = '\0';
+    f = s + cursor;
+    if (*f == '/') {
+	goto_error (f, 1);
+    } else {
+	char *t;
+	t = user_file_list_search (0, 0, 0, f);
+	if (t) {
+	    goto_error (t, 1);
+	    free (t);
+	}
+    }
+    free (s);
+}
+
 extern char *init_font;
 extern char *init_widget_font;
 extern char *init_bg_color;
@@ -1734,6 +1765,9 @@ static void main_loop (void)
 		    if (option_pull_down_window_list)
 			CPullDown (CIdent ("menu.wlist"));
 		    break;
+		case CK_Jump_To_File:
+		    menu_jump_to_file (0);
+		    break;
 		case CK_Menu:
 		    break;
 		default:
@@ -1964,10 +1998,12 @@ int main (int argc, char **argv)
     argv_nought = (char *) strdup (argv[0]);
 
 
+#ifdef LC_CTYPE
     if (!setlocale (LC_CTYPE, ""))
 	fprintf (stderr, _ ("%s: cannot set locale, see setlocale(3), current locale is %s\n"), argv_nought, setlocale (LC_MESSAGES, 0));
     setlocale (LC_TIME, "");
     setlocale (LC_MESSAGES, "");
+#endif
     bindtextdomain (PACKAGE, LOCALEDIR);
     textdomain (PACKAGE);
 
@@ -2024,7 +2060,7 @@ int main (int argc, char **argv)
 
     if (option_font)
 	if (!strcmp (option_font, "default"))
-	    option_font = (char *) strdup ("-*-fixed-bold-r-*--13-*-*-*-*-80-*");
+	    option_font = (char *) strdup ("-*-fixed-bold-r-*--13-120-*-*-*-80-*");
 
     if (option_widget_font)
 	if (!strcmp (option_widget_font, "default"))
@@ -2046,18 +2082,6 @@ int main (int argc, char **argv)
     cooledit_init ();
     main_window = CDrawMainWindow ("cooledit", "Cooledit");
     xdnd_set_dnd_aware (CDndClass, main_window, 0);
-    if (display_dnd_protocol_change_message) {
-	CMessageDialog (CRoot, 0, 0, 0, _("Cooledit Note"),
-			_("Cooledit has detected the option \"option_dnd_version\" in your\n" \
-			"user initialisation file ~/.cedit/.cooledit.ini under the section\n" \
-			"[Options]. This means you are upgrading from a version prior to\n" \
-			"3.7.0. Versions after 3.6.3 no longer support the Dnd protocol.\n" \
-	 "Only the XDND protocol is supported (version 0 through 2).\n" \
-			"Some minor behavioural changes will be evident. In particular,\n" \
-			"Coolicon drop scripts will NO LONGER WORK and should be rewritten\n" \
-	 "to take advantage of the XDND MIME data types. Consult the\n" \
-			"cooledit(1) and coolicon(1) man pages and the FAQ for further info.\n"));
-    }
     w = CWidgetOfWindow (main_window);
     w->funcs = mouse_funcs_new (w, &main_mouse_funcs);
 
