@@ -132,7 +132,7 @@ typedef struct struct_debug {
     char *file;
     char *condition;
     int show_output, stop_at_main, show_on_stdout;
-    int ignore_SIGTTOU;
+/*    int ignore_SIGTTOU; */
     pid_t xterm_pid;
     int break_point_line;
     char *break_point_editor;
@@ -142,7 +142,7 @@ extern CWidget *edit[50];
 extern int current_edit;
 extern int last_edit;
 
-static void debug_message (Debug * d, char *heading, char *str, int options)
+static void debug_message (Debug * d, const char *heading, const char *str, int options)
 {
     if (d->pid) {
 	CRemoveWatch (d->in, debug_write_callback, WATCH_WRITING);
@@ -156,7 +156,7 @@ static void debug_message (Debug * d, char *heading, char *str, int options)
     }
 }
 
-static void debug_error2 (Debug * d, char *par1, char *par2)
+static void debug_error2 (Debug * d, const char *par1, const char *par2)
 {
     char *t;
     if (d->pid) {
@@ -396,7 +396,7 @@ static int xdebug_goto_line (Debug * d, char *file, int line)
     return 1;
 }
 
-void goto_file_dialog (char *heading, char *tool_hint, char *text);
+void goto_file_dialog (const char *heading, const char *tool_hint, const char *text);
 
 static Debug debug_session;
 
@@ -489,12 +489,14 @@ static void xdebug_check_watch_point (Debug * d, char *m)
     free (s);
 }
 
+int writeall (int fd, char *buf, int len);
+
 static void debug_read_callback (int fd, fd_set * reading, fd_set * writing, fd_set * error, void *data)
 {
     Debug *d;
     char *p;
     char buf[1025];
-    int c, action, n;
+    int c, action, n, i;
     d = &debug_session;
     if (CChildExitted (d->pid, 0) || (!rxvt_have_pid (d->xterm_pid) && d->show_output)) {
 	debug_finish (0);
@@ -502,21 +504,13 @@ static void debug_read_callback (int fd, fd_set * reading, fd_set * writing, fd_
     }
     do {
 	c = read (d->out, buf, 1024);
-    } while (c < 0 && errno == EINTR);
-    if (c <= 0)
+    } while (c < 0 && (errno == EINTR || errno == EAGAIN));
+    if (c <= 0) {
+	debug_error1 (d, _ ("Error reading from the debugger"));
+	debug_finish (0L);
 	return;
-    buf[c] = '\0';
-    for (n = 0; d->query[n].query; n++) {
-	if (from_end (buf, d->query[n].query, 1)) {
-	    p = strrchr (buf, '\n');
-	    if (!p)
-		buf[0] = '\0';
-	    else
-		p[1] = '\0';
-	    while (write (d->in, d->query[n].response, strlen (d->query[n].response)) < 0 && errno == EINTR);
-	    break;
-	}
     }
+    buf[c] = '\0';
     n = 0;
     if (!d->pool)
 	d->pool = pool_init ();
@@ -525,6 +519,21 @@ static void debug_read_callback (int fd, fd_set * reading, fd_set * writing, fd_
 	printf ("%s", buf);
 	fflush (stdout);
     }
+    for (i = 0; d->query[i].query; i++) {
+	if (from_end ((char *) pool_start (d->pool), d->query[i].query, 1)) {
+	    pool_drop_last_line (d->pool);
+	    if (d->show_on_stdout) {
+		printf ("%s", d->query[i].response);
+		fflush (stdout);
+	    }
+	    if (writeall (d->in, d->query[i].response, strlen (d->query[i].response)) != strlen (d->query[i].response)) {
+		debug_error1 (d, _ ("Error writing to the debugger"));
+		debug_finish (0L);
+		return;
+	    }
+	    break;
+	}
+    }
     if (!d->action)
 	goto fin_read;
     p = from_end ((char *) pool_start (d->pool), d->prompt, 0);
@@ -550,7 +559,7 @@ static void debug_read_callback (int fd, fd_set * reading, fd_set * writing, fd_
     case ACTION_VERSION:
 /* 4.18 doesn't seem to do next's properly */
 	if (!strstr ((char *) pool_start (d->pool), GDB_VERSION1) && !strstr ((char *) pool_start (d->pool), GDB_VERSION3)) {
-	    static once = 1;
+	    static int once = 1;
 	    if (once) {
 /* NLS: <version> or <version> expected. You seem... */
 		debug_error1 (d, catstrs (GDB_VERSION1 " ",  _ ("or"), " " GDB_VERSION3, _ (" expected. You seem to have a different debugger \n or version. If things don't work properly, this is why. \n This error will not display again in this session. "), 0));
@@ -649,31 +658,57 @@ static void debug_read_callback (int fd, fd_set * reading, fd_set * writing, fd_
 	d->pool = 0;
 	break;
     case ACTION_INFO_SOURCE:
-	if (d->file) {
-	    free (d->file);
+    	{
+	    char *last_file;
+	    last_file = d->file;
 	    d->file = 0;
-	}
-	p = strstr ((char *) pool_start (d->pool), "Located in ");
-	if (p) {
-	    char *q;
-	    q = p + 11;
-	    p = strchr (q, '\n');
+	    p = strstr ((char *) pool_start (d->pool), "Located in ");
 	    if (p) {
-		*p = '\0';
-		d->file = (char *) strdup (q);
+		char *q;
+		q = p + 11;
+		p = strchr (q, '\n');
+		if (p) {
+		    *p = '\0';
+		    d->file = (char *) strdup (q);
+		}
+	    } else {	/* try to see if we are still in the same source file */
+
+/* ------ the functionality in this block is currently not useful ------ */
+		char *base_name;
+		p = strstr ((char *) pool_start (d->pool), "Current source file is ");
+		if (p) {
+		    base_name = p + 23;
+		    p = strchr (base_name, '\n');
+		    if (!p) {
+			base_name = 0;
+		    } else {
+			*p = '\0';
+			if (last_file) {
+			    p = (p = strrchr (last_file, '/')) ? p + 1 : last_file;	/* get the basename */
+			    if (!strcmp (base_name, p)) {
+				d->file = last_file;
+				last_file = 0;
+			    }
+			}
+		    }
+		}
+		if (!d->file && base_name)
+		    d->file = user_file_list_search (0, 0, 0, base_name);
 	    }
+	    if (last_file)
+		free (last_file);
+	    p = strstr ((char *) pool_start (d->pool), "No current source file");
+	    if (p) {
+		xdebug_cursor_bookmark_flush ();
+	    } else {
+		if (d->file && d->line)	/* if we can't get the file number, issue a backtrace */
+		    xdebug_goto_line (d, d->file, d->line);
+		else
+		    xdebug_append_command (d, d->get_backtrace, ACTION_BACKTRACE, 0, 0);
+	    }
+	    pool_free (d->pool);
+	    d->pool = 0;
 	}
-	p = strstr ((char *) pool_start (d->pool), "No current source file");
-	if (p) {
-	    xdebug_cursor_bookmark_flush ();
-	} else {
-	    if (d->file && d->line)	/* if we can't get the file number, issue a backtrace */
-		xdebug_goto_line (d, d->file, d->line);
-	    else
-		xdebug_append_command (d, d->get_backtrace, ACTION_BACKTRACE, 0, 0);
-	}
-	pool_free (d->pool);
-	d->pool = 0;
 	break;
     case ACTION_INFO_PROGRAM:
 	p = strstr ((char *) pool_start (d->pool), "Pid");
@@ -787,9 +822,11 @@ static void debug_write_callback (int fd, fd_set * reading, fd_set * writing, fd
     d->break_point_line = 0;
 /* no backtrace if no program running, all other commands are ok */
     if (!(!d->child && (d->command[0].action == ACTION_INFO_SOURCE || d->command[0].action == ACTION_BACKTRACE))) {
-	do {
-	    c = write (d->in, d->command[0].command, strlen (d->command[0].command));
-	} while (c < 0 && errno == EINTR);
+	if (writeall (d->in, d->command[0].command, strlen (d->command[0].command)) != strlen (d->command[0].command)) {
+	    debug_error1 (d, _ ("Error writing to the debugger"));
+	    debug_finish (0L);
+	    return;
+	}
 	d->action = d->command[0].action;
 	d->break_point_editor = d->command[0].editor;
 	d->break_point_line = d->command[0].line;
@@ -883,12 +920,14 @@ static void xdebug_run (Debug * d)
     xdebug_append_command (d, "b main\n", ACTION_BREAKPOINT, 0, 0);
     xdebug_append_command (d, "r\n", ACTION_RUNNING, 0, 0);
     xdebug_append_command (d, "clear\n", ACTION_BREAKPOINT, 0, 0);
+#if 0
 /* programs get this signal when they ioctl the terminal given to them by gdb */
 /* this prevents them quiting when using the `show output' option */
     if (d->ignore_SIGTTOU) {
 	sprintf (s, "call signal (%d, (void *) %lu)\n", SIGTTOU, (unsigned long) SIG_IGN);
 	xdebug_append_command (d, s, ACTION_PROMPT, 0, 0);
     }
+#endif
     xdebug_append_command (d, "info program\n", ACTION_INFO_PROGRAM, 0, 0);
     if (!d->stop_at_main)
 	xdebug_append_command (d, "c\n", ACTION_RUNNING, 0, 0);
@@ -975,7 +1014,9 @@ static int debug_set_info (unsigned long x)
 	gettext_noop ("Show program output"),
 	gettext_noop ("Stop at main() function"),
 	gettext_noop ("Show on stdout"),
+#if 0
 	gettext_noop ("Ignore SIGTTOU"),
+#endif
 	0
     };
     char *check_tool_hints[10] =
@@ -983,7 +1024,9 @@ static int debug_set_info (unsigned long x)
 	gettext_noop ("This will cause the program to be opened udner an rxvt terminal so that its output can be viewed."),
 	gettext_noop ("This will cause the debugger to break at the first\nline of code in the program when issuing a `Start'."),
 	gettext_noop ("This will cause all debugger commands to show at\nthe terminal where you started cooledit."),
+#if 0
 	gettext_noop ("Run `call signal()' inside gdb so that SIGTTOU signals are ignored. Without this gdb sometimes exits."),
+#endif
 	0
     };
     char *input_names[10] =
@@ -1007,8 +1050,12 @@ static int debug_set_info (unsigned long x)
     checks_values_result[0] = &debug_session.show_output;
     checks_values_result[1] = &debug_session.stop_at_main;
     checks_values_result[2] = &debug_session.show_on_stdout;
+#if 0
     checks_values_result[3] = &debug_session.ignore_SIGTTOU;
     checks_values_result[4] = 0;
+#else
+    checks_values_result[3] = 0;
+#endif
     r = CInputsWithOptions (0, 0, 0, _ (" Debug : Program Data "), inputs_result, input_labels, input_names, input_tool_hint, checks_values_result, check_labels, check_tool_hints, INPUTS_WITH_OPTIONS_BROWSE_LOAD_1, 60);
     if (r)
 	return 1;
@@ -1260,7 +1307,9 @@ void debug_init (void)
     debug_session.query[3].response = 0;
     debug_session.debugger = "cooledit-gdb";
     debug_session.stop_at_main = 1;
+#if 0
     debug_session.ignore_SIGTTOU = 1;
+#endif
     xdebug_init_animation ();
 }
 
