@@ -62,11 +62,11 @@
 #define MID_X 20
 #define MID_Y 20
 
-int execute_background_display_output (char *title, char *s, char *name);
+int execute_background_display_output (const char *title, const char *s, const char *name);
 
 Window find_mapped_window (Window w);
 
-int CInputsWithOptions (Window parent, int x, int y, char *heading, \
+int CInputsWithOptions (Window parent, int x, int y, const char *heading, \
 	    char **inputs[], char *input_labels[], char *input_names[], \
     char *input_tool_hint[], int *check_values[], char *check_labels[], \
 			char *check_tool_hints[], int options, int width)
@@ -103,13 +103,11 @@ int CInputsWithOptions (Window parent, int x, int y, char *heading, \
 	hot = strchr (l, '&');
 	if (!hot)
 	    hot = "";
-	else {
-	    strcpy (hot, hot + 1);
-	}
+	else
+	    memmove (hot, hot + 1, strlen (hot + 1) + 1);
 	id[7] = 'A' + k;
 	CGetHintPos (0, &yh);
 	(CDrawText (id, win, xh, yh, _ (l)))->hotkey = *hot;
-	free (l);
 	CGetHintPos (0, &yh);
 	last = CLastInput (input_names[k]);
 	if (!*last || input_names[k][0] == '~')
@@ -133,6 +131,7 @@ int CInputsWithOptions (Window parent, int x, int y, char *heading, \
 	    CSetToolHint (id, _ (input_tool_hint[k]));
 	    CSetToolHint (input_names[k], _ (input_tool_hint[k]));
 	}
+	free (l);
     }
 
     for (k = 0; check_values[k] && check_labels[k]; k++);
