@@ -292,8 +292,6 @@ static struct {
 
 int save_options_section (const char *file, const char *section, const char *text);
 
-extern int display_dnd_protocol_change_message;
-
 int load_setup (const char *file)
 {
     static char *options_section = 0;
@@ -308,9 +306,6 @@ int load_setup (const char *file)
 	return 0;
 
     options_section = get_options_section (file, "[Options]");
-    if (options_section)
-	if (strstr (options_section, "option_dnd_version"))
-	    display_dnd_protocol_change_message = 1;
     p = q = options_section;
 
     if (!options_section)
@@ -521,7 +516,7 @@ void draw_options_dialog (Window parent, int x, int y)
 	    CGetHintPos (&x, 0);
 	    w = CDrawTextInput (short_name (integer_options[i].name, ""), win, x, y, FONT_MEAN_WIDTH * 8 + FONT_PIX_PER_LINE + 4, AUTO_HEIGHT, 8, itoa (*integer_options[i].value));
 	    w->position |= POSITION_FILL;
-	    w->label = _(integer_options[i].prompt);
+	    w->label = (char *) _(integer_options[i].prompt);        /* FIXME: memory leak? should not label be free'd? */
 	    w->hotkey = find_hotkey (w);
 	    w->label = 0;
 	    wdt->hotkey = w->hotkey;
