@@ -45,7 +45,9 @@
 
 
 
-int last_region = 0;
+static int last_region = 0;
+
+static int run_callbacks (CWidget * w, XEvent * xevent, CEvent * cwevent);
 
 
 int (*global_alarm_callback[33]) (CWidget *, XEvent *, CEvent *) = {
@@ -743,9 +745,9 @@ int num_event_names = sizeof (event_names) / sizeof (num_event_names);
 
 #endif
 
-int run_callbacks (CWidget * w, XEvent * xevent, CEvent * cwevent)
+static int run_callbacks (CWidget * w, XEvent * xevent, CEvent * cwevent)
 {
-    static char no_ident[33];
+    static char no_ident[33] = "";
     int handled = 0;
 
     if (!cwevent->text)
@@ -758,11 +760,25 @@ int run_callbacks (CWidget * w, XEvent * xevent, CEvent * cwevent)
     printf ("Calling %s: Type=%s\n", w->ident, type < num_event_names ? event_names[type] : itoa (type));
 #endif
     if (w->eh) {
-	if (w->callback_before)
+	char ident[33];
+	int (*cb) (struct cool_widget *, XEvent *, CEvent *);
+	cb = w->callback;
+	strcpy (ident, w->ident);
+	if (w->callback_before) {
 	    handled |= (*(w->callback_before)) (w, xevent, cwevent);
+	    if (w != CIdent (ident))	/* callback
+	    could free widget. hack this line must go
+	    here because ident could = "". so happens
+	    that such widgets don't have callbacks */
+		return handled;
+	}
 	handled |= (*(w->eh)) (w, xevent, cwevent);
-	if (w->callback && cwevent->ident[0])
-	    handled |= (*(w->callback)) (w, xevent, cwevent);
+	if (cb) {
+	    if (w != CIdent (ident))	/* same logic */
+		return handled;
+	    if (cwevent->ident[0])
+		handled |= (*(w->callback)) (w, xevent, cwevent);
+	}
     }
 #ifdef DEBUG_ENTRY
 /* NLS ? */
@@ -905,7 +921,7 @@ static void show_toolhint (Window win, int xp, int yp)
     t->label = (char *) strdup (w->toolhint);
 }
 
-void CSetToolHint (const char *ident, char *text)
+void CSetToolHint (const char *ident, const char *text)
 {
     CWidget *w;
     if (!text)
@@ -928,18 +944,21 @@ void CSetToolHint (const char *ident, char *text)
 #define FD_SETSIZE 256
 #endif
 
-static struct file_des_watch {
+struct file_des_watch {
     int fd, how;
     void (*callback) (int, fd_set *, fd_set *, fd_set *, void *);
-    struct file_des_watch *next;
+    const char *file;
+    int line;
     void *data;
-} *watch_table[FD_SETSIZE];
+};
+
+static struct file_des_watch *watch_table[FD_SETSIZE];
 
 static int watch_table_last = 0;
 
 /* returns non-zero if table is full */
-int CAddWatch (int fd, void (*callback) (int, fd_set *, fd_set *, fd_set *, void *), int how,
-	       void *data)
+int _CAddWatch (char *file, int line, int fd, void (*callback) (int, fd_set *, fd_set *, fd_set *, void *),
+		int how, void *data)
 {
     int i;
     if (!callback || fd < 0 || !how) {
@@ -964,6 +983,8 @@ int CAddWatch (int fd, void (*callback) (int, fd_set *, fd_set *, fd_set *, void
     watch_table[i]->how = how;
     watch_table[i]->fd = fd;
     watch_table[i]->data = data;
+    watch_table[i]->file = file;
+    watch_table[i]->line = line;
     if (watch_table_last < i + 1)
 	watch_table_last = i + 1;
     return 0;
@@ -1068,6 +1089,7 @@ static int run_watches (void)
 		continue;
 	    fd = watch_table[i]->fd;
 	    if (FD_ISSET (fd, &reading) && watch_table[i]->how & WATCH_READING) {
+/* printf ("check watch %s:%d\n", watch_table[i]->file, watch_table[i]->line); */
 		(*watch_table[i]->callback) (fd, &reading, &writing, &error, watch_table[i]->data);
 		FD_CLR (fd, &reading);
 		found_watch = 1;
