@@ -105,6 +105,25 @@ char *edit_get_buffer_as_text (WEdit * e)
     return t;
 }
 
+/* returns a buffer containing the current line and the offset of the
+cursor from the left margin in bytes: */
+char *edit_get_current_line_as_text (WEdit * e, long *length, long *cursor)
+{
+    long start, end;
+    char *r, *p;
+    start = edit_bol (e, e->curs1);
+    end = edit_eol (e, e->curs1);
+    if (length)
+	*length = end - start;
+    if (cursor)
+	*cursor = e->curs1 - start;
+    p = r = (char *) CMalloc (end - start + 1);
+    while (start < end)
+	*p++ = edit_get_byte (e, start++);
+    *p = '\0';
+    return r;
+}
+
 
 /* Note on CRLF->LF translation: */
 
@@ -878,7 +897,7 @@ void edit_insert_ahead (WEdit * edit, int c)
 	edit->force |= REDRAW_AFTER_CURSOR;
     }
     edit_modification (edit, edit->curs1);
-    edit_push_action (edit, DELETE);
+    edit_push_action (edit, ACT_DELETE);
 
     edit->mark1 += (edit->mark1 >= edit->curs1);
     edit->mark2 += (edit->mark2 >= edit->curs1);
@@ -1882,7 +1901,7 @@ void edit_do_undo (WEdit * edit)
 	case BACKSPACE:
 	    edit_backspace (edit);
 	    break;
-	case DELETE:
+	case ACT_DELETE:
 	    edit_delete (edit);
 	    break;
 	case COLUMN_ON:
@@ -2714,6 +2733,7 @@ int edit_execute_cmd (WEdit * edit, int command, long char_for_insertion)
     case CK_Cancel:
     case CK_Save_Desktop:
     case CK_New_Window:
+    case CK_Jump_To_File:
     case CK_Cycle:
     case CK_Save_And_Quit:
     case CK_Check_Save_And_Quit:
