@@ -177,6 +177,7 @@
 #define SEARCH_DIALOG_OPTION_BACKWARDS	8
 #define SEARCH_DIALOG_OPTION_BOOKMARK	16
 
+#define FILELIST_FILE "/.cedit/filelist"
 #define SYNTAX_FILE "/.cedit/Syntax"
 #define CLIP_FILE "/.cedit/cooledit.clip"
 #define MACRO_FILE "/.cedit/cooledit.macros"
@@ -257,7 +258,7 @@
 /*some codes that may be pushed onto or returned from the undo stack: */
 #define CURS_LEFT 601
 #define CURS_RIGHT 602
-#define DELETE 603
+#define ACT_DELETE 603
 #define BACKSPACE 604
 #define STACK_BOTTOM 605
 #define CURS_LEFT_LOTS 606
@@ -286,11 +287,7 @@ struct selection {
 struct mb_rule {
     long ch;
     char end;
-#ifdef HAVE_WCHAR_H
     mbstate_t shift_state;
-#else
-    long shift_state;
-#endif
 };
 
 struct _mb_marker {
@@ -485,7 +482,7 @@ void edit_draw_menus (Window parent, int x, int y);
 void edit_run_make (void);
 void edit_change_directory (void);
 int edit_man_page_cmd (WEdit * edit);
-void edit_search_replace_dialog (Window parent, int x, int y, char **search_text, char **replace_text, char **arg_order, char *heading, int option);
+void edit_search_replace_dialog (Window parent, int x, int y, char **search_text, char **replace_text, char **arg_order, const char *heading, int option);
 void edit_search_dialog (WEdit * edit, char **search_text);
 long edit_find (long search_start, unsigned char *expr, int *len, long last_byte, int (*get_byte) (void *, long), void *data, void *d);
 void edit_set_foreground_colors (unsigned long normal, unsigned long bold, unsigned long italic);
@@ -505,7 +502,7 @@ void edit_menu_cmd (WEdit * e);
 void edit_init_menu_emacs (void);
 void edit_init_menu_normal (void);
 void edit_done_menu (void);
-int edit_raw_key_query (char *heading, char *query, int cancel);
+int edit_raw_key_query (const char *heading, const char *query, int cancel);
 char *strcasechr (const unsigned char *s, int c);
 int edit (const char *_file, int line);
 int edit_translate_key (WEdit * edit, unsigned int x_keycode, long x_key, int x_state, int *cmd, int *ch);
@@ -533,6 +530,7 @@ static inline int edit_get_byte (WEdit * edit, long byte_index)
 #endif
 
 char *edit_get_buffer_as_text (WEdit * edit);
+char *edit_get_current_line_as_text (WEdit * e, long *length, long *cursor);
 int edit_count_lines (WEdit * edit, long current, int upto);
 long edit_move_forward (WEdit * edit, long current, int lines, long upto);
 long edit_move_forward3 (WEdit * edit, long current, int cols, long upto);
@@ -601,6 +599,7 @@ void edit_help_cmd (WEdit * edit);
 void edit_left_word_move (WEdit * edit, int s);
 void edit_right_word_move (WEdit * edit, int s);
 void edit_get_selection (WEdit * edit);
+char *edit_get_text_from_selection_history (Window parent, int x, int y, int cols, int lines, long *len);
 
 int edit_save_macro_cmd (WEdit * edit, struct macro macro[], int n);
 int edit_load_macro_cmd (WEdit * edit, struct macro macro[], int *n, int k);
