@@ -204,32 +204,19 @@ void CRefreshEditor (WEdit * edit)
 
 #endif
 
-#ifndef MIDNIGHT
-#ifndef GTK
+#define DEFAULT_CREATE_MODE		(S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)
 
 /* three argument open */
-int my_open (const char *pathname, int flags,...)
+int open_create (const char *pathname, int flags, mode_t mode)
 {
     int file;
-    va_list ap;
-
     file = open ((char *) pathname, O_RDONLY);
-    if (file < 0 && (flags & O_CREAT)) {	/* must it be created ? */
-	mode_t mode;
-	va_start(ap, flags);
-	mode = va_arg(ap, mode_t);
-	va_end(ap);
+    if (file < 0 && (flags & O_CREAT))	/* must it be created ? */
 	return creat ((char *) pathname, mode);
-    }
     close (file);
     return open ((char *) pathname, flags);
 }
 
-#define open my_open
-
-#endif
-#endif
-
 /* "Oleg Yu. Repin" <repin@ssd.sscc.ru> added backup filenames
     ...thanks -paul */
 
@@ -257,7 +244,7 @@ int edit_save_file (WEdit * edit, const char *filename)
 
     savename = (char *) strdup ((char *) filename);
 
-    if ((fd = open (savename, O_WRONLY)) == -1) {
+    if ((fd = open_create (savename, O_WRONLY, DEFAULT_CREATE_MODE)) == -1) {
 	this_save_mode = 0;	/* the file does not exists yet, so no safe save or backup necessary */
     } else {
 	close (fd);
@@ -534,7 +521,7 @@ int edit_save_as_cmd (WEdit * edit)
 	    if (strcmp(catstrs (edit->dir, edit->filename, 0), exp)) {
 		int file;
 		different_filename = 1;
-		if ((file = open ((char *) exp, O_RDONLY)) != -1) {	/* the file exists */
+		if ((file = open_create ((char *) exp, O_RDONLY, DEFAULT_CREATE_MODE)) != -1) {	/* the file exists */
 		    close (file);
 		    if (edit_query_dialog2 (_(" Warning "), 
 		    _(" A file already exists with this name. "), 
@@ -621,13 +608,13 @@ int edit_raw_key_query (char *heading, char *query, int cancel)
 
 #else
 
-int edit_raw_key_query (char *heading, char *query, int cancel)
+int edit_raw_key_query (const char *heading, const char *query, int cancel)
 {
 #ifdef GTK
     /* *** */
     return 0;
 #else
-    return CKeySymMod (CRawkeyQuery (0, 0, 0, heading, query));
+    return CKeySymMod (CRawkeyQuery (0, 0, 0, heading, "%s", query));
 #endif
 }
 
@@ -639,7 +626,7 @@ static FILE *edit_open_macro_file (const char *r)
     char *filename;
     int file;
     filename = catstrs (home_dir, MACRO_FILE, 0);
-    if ((file = open (filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) == -1)
+    if ((file = open_create (filename, O_CREAT | O_RDWR, DEFAULT_CREATE_MODE)) == -1)
 	return 0;
     close (file);
     return fopen (filename, r);
@@ -764,7 +751,7 @@ void edit_delete_macro_cmd (WEdit * edit)
 /* *** */
     command = 0;
 #else
-    command = CKeySymMod (CRawkeyQuery (0, 0, 0, _ (" Delete Macro "),
+    command = CKeySymMod (CRawkeyQuery (0, 0, 0, _ (" Delete Macro "), "%s",
 /* Input line for a single key press follows the ':' */
 					_ (" Press macro hotkey: ")));
 #endif
@@ -983,13 +970,15 @@ void edit_block_copy_cmd (WEdit * edit)
     int size, x;
     unsigned char *copy_buf;
 
-    edit_update_curs_col (edit);
-    x = edit->curs_col;
     if (eval_marks (edit, &start_mark, &end_mark))
 	return;
-    if (column_highlighting)
-	if ((x >= edit->column1 && x < edit->column2) || (x > edit->column2 && x <= edit->column1))
-	    return;
+    if (column_highlighting) {
+	edit_update_curs_col (edit);
+	x = edit->curs_col;
+	if (start_mark <= edit->curs1 && end_mark >= edit->curs1)
+	    if ((x > edit->column1 && x < edit->column2) || (x > edit->column2 && x < edit->column1))
+		return;
+    }
 
     copy_buf = edit_get_block (edit, start_mark, end_mark, &size);
 
@@ -1470,7 +1459,7 @@ void edit_indent_left_right_paragraph (WEdit * edit)
 extern struct look *look;
 
 void edit_search_replace_dialog (Window parent, int x, int y, char **search_text,
-				 char **replace_text, char **arg_order, char *heading, int option)
+				 char **replace_text, char **arg_order, const char *heading, int option)
 {
     (*look->search_replace_dialog) (parent, x, y, search_text, replace_text, arg_order, heading, option);
 }
@@ -1613,6 +1602,7 @@ int string_regexp_search (char *pattern, char *string, int len, int match_type,
 	    free (old_pattern);
 	    old_pattern = 0;
 	}
+	memset (&r, '\0', sizeof (r));
 	if (regcomp (&r, pattern, REG_EXTENDED | (icase ? REG_ICASE : 0))) {
 	    *found_len = 0;
 	    return -3;
@@ -2356,7 +2346,7 @@ int edit_save_block (WEdit * edit, const char *filename, long start, long finish
 {
     int len, file;
 
-    if ((file = open ((char *) filename, O_CREAT | O_WRONLY | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)) == -1)
+    if ((file = open_create ((char *) filename, O_CREAT | O_WRONLY | O_TRUNC, DEFAULT_CREATE_MODE)) == -1)
 	return 0;
 
     if (column_highlighting) {
@@ -2464,36 +2454,39 @@ char *selection_get_line (void *data, int line)
     return (char *) t;
 }
 
-void edit_paste_from_history (WEdit * edit)
+char *edit_get_text_from_selection_history (Window parent, int x, int y, int cols, int lines, long *len)
 {
-    int i, c;
-
-    edit_update_curs_col (edit);
-    edit_update_curs_row (edit);
-
-    c = max (20, edit->num_widget_columns - 5);
-
+    int i;
 #ifdef GTK
 #if 0
 /* *** */
-    i = gtk_edit_list_box_dialog (c, 10,
-	       0, NUM_SELECTION_HISTORY - 10, NUM_SELECTION_HISTORY - 1, NUM_SELECTION_HISTORY,
-	       selection_get_line, (void *) selection_history);
+    i = gtk_edit_list_box_dialog (c, 10, 0, NUM_SELECTION_HISTORY - 10, NUM_SELECTION_HISTORY - 1,
+				  NUM_SELECTION_HISTORY, selection_get_line, (void *) selection_history);
 #else
     i = -1;
 #endif
 #else
-    i = CListboxDialog (WIN_MESSAGES, c, 10,
-	       0, NUM_SELECTION_HISTORY - 10, NUM_SELECTION_HISTORY - 1, NUM_SELECTION_HISTORY,
-	       selection_get_line, (void *) selection_history);
+    i = CListboxDialog (parent, x, y, cols, lines, 0, NUM_SELECTION_HISTORY - lines, NUM_SELECTION_HISTORY - 1,
+			NUM_SELECTION_HISTORY, selection_get_line, (void *) selection_history);
 #endif
-
-    if (i < 0)
-	return;
-
+    if (i < 0) {
+	*len = 0;
+	return NULL;
+    }
     i = (current_selection + i + 1) % NUM_SELECTION_HISTORY;
+    *len = selection_history[i].len;
+    return selection_history[i].text;
+}
 
-    paste_text (edit, selection_history[i].text, selection_history[i].len);
+void edit_paste_from_history (WEdit * edit)
+{
+    long len;
+    char *s;
+
+    edit_update_curs_col (edit);
+    edit_update_curs_row (edit);
+    s = edit_get_text_from_selection_history (WIN_MESSAGES, max (20, edit->num_widget_columns - 5), 10, &len);
+    paste_text (edit, s, len);
     edit->force |= REDRAW_COMPLETELY;
 }
 
