@@ -41,7 +41,7 @@ extern int EditExposeRedraw;
 CWidget *wedit = 0;
 
 void shell_output_kill_jobs (WEdit * edit);
-void shell_output_kill_job (WEdit * edit, pid_t pid);
+void shell_output_kill_job (WEdit * edit, pid_t pid, int send_term);
 
 void edit_destroy_callback (CWidget * w)
 {
@@ -88,7 +88,7 @@ static void shell_output_write_callback (int fd, fd_set * reading, fd_set * writ
 	for (j = edit->jobs; j; j = j->next)
 	    if (j->in == fd || j->out == fd) {
 		if (CChildExitted (j->pid, 0))
-		    shell_output_kill_job (edit, j->pid);
+		    shell_output_kill_job (edit, j->pid, 0);
 		break;
 	    }
 	return;
@@ -117,7 +117,7 @@ static void shell_output_read_callback (int fd, fd_set * reading, fd_set * writi
 	for (j = edit->jobs; j; j = j->next)
 	    if (j->in == fd || j->out == fd) {
 		if (CChildExitted (j->pid, 0))
-		    shell_output_kill_job (edit, j->pid);
+		    shell_output_kill_job (edit, j->pid, 0);
 		break;
 	    }
 	return;
@@ -153,7 +153,7 @@ void shell_output_add_job (WEdit * edit, int in, int out, pid_t pid, char *name,
     edit->jobs = j;
 }
 
-static void shell_output_destroy_job (struct shell_job *j)
+static void shell_output_destroy_job (struct shell_job *j, int send_term)
 {
     if (j->out >= 0) {
 	CRemoveWatch (j->out, shell_output_read_callback, WATCH_READING);
@@ -163,8 +163,10 @@ static void shell_output_destroy_job (struct shell_job *j)
 	CRemoveWatch (j->in, shell_output_write_callback, WATCH_WRITING);
 	close (j->in);
     }
-    if (j->pid > 0)
-	kill (j->pid, SIGTERM);
+    if (send_term && j->pid > 0) {
+	if (!kill (j->pid, SIGTERM))
+	    CChildWait (j->pid);
+    }
     free (j->name);
     memset (j, 0, sizeof (*j));
     free (j);
@@ -177,12 +179,12 @@ void shell_output_kill_jobs (WEdit * edit)
 	return;
     for (j = edit->jobs; j; j = n) {
 	n = j->next;
-	shell_output_destroy_job (j);
+	shell_output_destroy_job (j, 1);
     }
     edit->jobs = 0;
 }
 
-void shell_output_kill_job (WEdit * edit, pid_t pid)
+void shell_output_kill_job (WEdit * edit, pid_t pid, int send_term)
 {
     struct shell_job *j, *p;
     int done;
@@ -193,12 +195,12 @@ void shell_output_kill_job (WEdit * edit, pid_t pid)
 	    if (j->pid == pid) {
 		if ((unsigned long) j == (unsigned long) edit->jobs) {
 		    edit->jobs = edit->jobs->next;
-		    shell_output_destroy_job (j);
+		    shell_output_destroy_job (j, send_term);
 		    done = 0;
 		    break;
 		} else {
 		    p->next = j->next;
-		    shell_output_destroy_job (j);
+		    shell_output_destroy_job (j, send_term);
 		    done = 0;
 		    break;
 		}
