@@ -311,7 +311,6 @@ static void focus_widget (CWidget * w)
 
 /* focus the new widget */
     CSendMessage (w, FocusIn);
-
 }
 
 static void focus_main (Window main, int type)
@@ -400,6 +399,36 @@ void CFocusNormal (CWidget * w)
     }
 }
 
+extern int option_never_raise_wm_windows;
+
+int CTryFocus (CWidget * w, int raise_wm_window)
+{
+    if (!option_never_raise_wm_windows) {
+	CFocus (w);
+	if (raise_wm_window)
+	    CRaiseWMWindow (w->ident);
+	return 1;
+    } else {
+/* focus only if it does not involve switching focus of a WM window: */
+	CWidget *v;
+	Window *win;
+	v = CWidgetOfWindow (CGetFocus ());
+	if (v)
+	    if (v->mainid == w->mainid) {
+		CFocus (w);
+		return 1;
+	    }
+#ifdef FOCUS_DEBUG
+	printf ("setting last_focussed to %s\n", w->ident);
+#endif
+	win = get_last_focussed_in_main (w->mainid);
+	if (*win)
+	    add_to_focus_stack (*win);
+	add_to_focus_stack ((*win = w->winid));
+    }
+    return 0;
+}
+
 void CFocusDebug (CWidget *w, int line, char *file)
 {
 /* NLS ? */
