@@ -94,7 +94,7 @@ CInitData *given = 0;
 /* defaults */
 #define DEFAULT_DISPLAY			NULL
 #define DEFAULT_GEOM			NULL
-#define DEFAULT_FONT			"-*-fixed-bold-r-*--13-*-*-*-*-80-*"
+#define DEFAULT_FONT			"-*-fixed-bold-r-*--13-120-*-*-*-80-*"
 #define FONT_FALLBACK			"-*"
 #define DEFAULT_BG_COLOR		0 /* not used *************/ 
 #define DEFAULT_WIDGET_COLOR_R		"0.9"
@@ -875,79 +875,115 @@ static RETSIGTYPE alarmhandler (int x)
 #endif
 }
 
-#define CHILD_EXITED_MAX	32
+#define CHILD_EXITED_MAX	256
 
 static struct {
     pid_t pid;
     int status;
-    unsigned long count;
 } children_exitted[CHILD_EXITED_MAX];
 
-static long child_exit_counter = 1;
-static int num_children_exitted = 0;
+static unsigned char children_exitted_leader = 0;
+static unsigned char children_exitted_trailer = 0;
 
 #if (RETSIGTYPE==void)
 #define handler_return return
 #else
-#define handler_return return 1
+#define handler_return return 0
 #endif
 
 static RETSIGTYPE childhandler (int x)
 {
-    num_children_exitted++;
+    int save_errno = errno;
+    pid_t pid;
+    pid = waitpid (-1, &children_exitted[children_exitted_leader].status, WNOHANG);
+    if (pid > 0) {
+	if ((unsigned char) (children_exitted_leader - children_exitted_trailer) <
+	    (unsigned char) ((int) CHILD_EXITED_MAX - 2)) {
+	    children_exitted[children_exitted_leader].pid = pid;
+	    children_exitted_leader++;
+	}
+    }
+    errno = save_errno;
     signal (SIGCHLD, childhandler);
     handler_return;
 }
 
-void  childhandler_ (void)
+struct child_exitted_item {
+    struct child_exitted_item *next;
+    pid_t pid;
+    int status;
+};
+
+struct child_exitted_list {
+    struct child_exitted_item *next;
+};
+
+static struct child_exitted_list child_list = {NULL};
+
+void childhandler_ (void)
 {
-    pid_t p, save_errno = errno;
-    int i, status, j, n;
-    if (!num_children_exitted)
-	return;
-    n = num_children_exitted + 1;
-    for (j = 0; j < n; j++) {
-	do {
-	    errno = 0;
-	} while ((-1 == (p = waitpid (-1, NULL, WNOHANG))) && (errno == EINTR));
-	if (num_children_exitted > 0)
-	    num_children_exitted--;
-	if (p > 0) {
-	    unsigned long minimum = (unsigned long) (-1);
-	    int i_min = 0;
-	    for (i = 0; i < CHILD_EXITED_MAX; i++) {
-		if (!children_exitted[i].pid) {
-		    i_min = i;
-		    break;
-		}
-		if (children_exitted[i].count < minimum) {
-		    minimum = children_exitted[i].count;
-		    i_min = i;
-		}
-	    }
-	    children_exitted[i_min].pid = p;
-	    children_exitted[i_min].status = status;
-	    children_exitted[i_min].count = child_exit_counter++;
-	}
+    while (children_exitted_trailer != children_exitted_leader) {
+        struct child_exitted_item *c;
+        c = malloc (sizeof (struct child_exitted_item));
+        memset (c, '\0', sizeof (*c));
+        c->pid = children_exitted[children_exitted_trailer].pid;
+        c->status = children_exitted[children_exitted_trailer].status;
+        c->next = child_list.next;
+        child_list.next = c;
+        children_exitted_trailer++;
     }
-    errno = save_errno;
 }
 
+#if 0
+/* returns non-zero on child exit */
+int CChildExitted (pid_t p, int *status)
+{
+    unsigned char i;
+    for (i = children_exitted_trailer; i != children_exitted_leader; i++)
+        if (p && children_exitted[i].pid == p) {
+            if (status)
+                *status = children_exitted[i].status;
+            children_exitted[i].pid = 0;
+            children_exitted[i].status = 0;
+            return 1;
+        }
+    return 0;
+}
+#endif
+
 /* returns non-zero on child exit */
 int CChildExitted (pid_t p, int *status)
 {
-    int i;
-    for (i = 0; i < CHILD_EXITED_MAX; i++)
-	if (p && children_exitted[i].pid == p) {
+    struct child_exitted_item *c;
+    if (!p)
+        return 0;
+    for (c = (struct child_exitted_item *) &child_list; c->next;) {
+        if (c->next->pid == p) {
+            struct child_exitted_item *t;
+            t = c->next;
+            c->next = c->next->next;
 	    if (status)
-		*status = children_exitted[i].status;
-	    children_exitted[i].pid = 0;
-	    children_exitted[i].count = 0;
-	    return 1;
-	}
+        	*status = t->status;
+            free (t);
+            return 1;
+        } else {
+            c = c->next;
+        }
+    }
     return 0;
 }
 
+void CChildWait (pid_t p)
+{
+    while (!CChildExitted (p, NULL)) {
+        struct timeval tv;
+        childhandler_ ();
+        tv.tv_sec = 0;
+        tv.tv_usec = 50000;
+        select (0, NULL, NULL, NULL, &tv);
+    }
+}
+
 static void set_child_handler (void)
 {
     memset (children_exitted, 0, sizeof (children_exitted));
