@@ -46,7 +46,7 @@ extern int replace_case;
 extern int replace_backwards;
 extern int search_create_bookmark;
 
-void look_cool_search_replace_dialog (Window parent, int x, int y, char **search_text, char **replace_text, char **arg_order, char *heading, int option)
+void look_cool_search_replace_dialog (Window parent, int x, int y, char **search_text, char **replace_text, char **arg_order, const char *heading, int option)
 {
     Window win;
     XEvent xev;
@@ -509,7 +509,7 @@ static Window draw_file_browser (const char *identifier, Window parent, int x, i
     CSetMovement (catstrs (identifier, ".dbox.vsc", 0), POSITION_HEIGHT | POSITION_RIGHT);
     CSetMovement (catstrs (identifier, ".dbox.hsc", 0), POSITION_RIGHT | POSITION_BOTTOM);
     CGetHintPos (&x2, &y2);
-    (CDrawText (catstrs (identifier, ".msg", 0), win, x, y2, ""))->position |= POSITION_FILL | POSITION_BOTTOM;
+    (CDrawText (catstrs (identifier, ".msg", 0), win, x, y2, _("Ctrl-Tab to complete, Alt-Ins for clip history, Shift-Up for history")))->position |= POSITION_FILL | POSITION_BOTTOM;
     CGetHintPos (0, &y2);
     (w = CDrawTextInput (catstrs (identifier, ".finp", 0), win, x, y2,
 		    WIDGET_SPACING * 2 - 2, AUTO_HEIGHT, 256, file))->position |= POSITION_FILL | POSITION_BOTTOM;
@@ -597,7 +597,7 @@ void input_insert (CWidget * w, int c);
 static char *file_error (void)
 {
     if (errno) {
-	char *error_msg;
+	const char *error_msg;
 #ifdef HAVE_STRERROR
 	error_msg = _ (strerror (errno));
 #else
