@@ -35,7 +35,7 @@
 
 extern struct look *look;
 
-extern void look_cool_search_replace_dialog (Window parent, int x, int y, char **search_text, char **replace_text, char **arg_order, char *heading, int option);
+extern void look_cool_search_replace_dialog (Window parent, int x, int y, char **search_text, char **replace_text, char **arg_order, const char *heading, int option);
 
 extern CWidget *look_cool_draw_file_list (const char *identifier, Window parent, int x, int y,
 					 int width, int height, int line, int column,
