@@ -554,6 +554,23 @@ void CInsertMenuItem (const char *ident, const char *after, const char *text, in
     insert_menu_item (w, i, text, hot_key, call_back, data);
 }
 
+void CInsertMenuItemAfter (const char *ident, const char *after, const char *text, int hot_key, callfn call_back, unsigned long data)
+{
+    int i;
+    CWidget *w;
+    w = CIdent (ident);
+    if (!w) {
+	CErrorDialog (0, 0, 0, _ (" Insert Menu Item "), " %s: %s ", _ ("No such menu"), ident);
+	return;
+    }
+    i = CHasMenuItem (ident, after);
+    if (i < 0) {
+	CErrorDialog (0, 0, 0, _ (" Insert Menu Item "), " %s: %s ", _ ("No such item"), after);
+	return;
+    }
+    insert_menu_item (w, i + 1, text, hot_key, call_back, data);
+}
+
 static void remove_item (CWidget * w, int i)
 {
     if (!w)
