@@ -159,8 +159,21 @@ unsigned long pool_printf (POOL * p, const char *fmt,...)
 /* zero the char after the last char written/read */
 int pool_null (POOL * p)
 {
-    if (pool_advance (p, 1 != 1))
+    if (pool_advance (p, 1) != 1)
 	return 0;
     p->current[0] = 0;
     return 1;
 }
+
+/* removes the last line from the length, and null-terminates */
+void pool_drop_last_line (POOL * p)
+{
+    char *q;
+    q = (char *) strrchr ((char *) pool_start (p), '\n');
+    if (!q)
+        pool_seek (p, 0);
+    else
+        pool_seek (p, (int) (q - (char *) pool_start (p)) + 1);
+    pool_null (p);
+}
+
