@@ -81,6 +81,9 @@ unsigned long mad_pool_advance (POOL * p, unsigned long l, char *file, int line)
 unsigned long pool_advance (POOL * p, unsigned long l);
 #endif
 
+/* removes the last line from the length, and null-terminates */
+void pool_drop_last_line (POOL * p);
+
 /* zero the char after the last char written/read without advancing current */
 int pool_null (POOL *p);
 
