@@ -1072,6 +1072,8 @@ static int edit_check_spelling_at (WEdit * edit, long byte_index)
 
 char *option_alternate_dictionary = "";
 
+int PATH_search (const char *file);
+
 int edit_check_spelling (WEdit * edit)
 {
     if (!option_auto_spellcheck)
@@ -1091,23 +1093,42 @@ int edit_check_spelling (WEdit * edit)
 /* is ispell running? */
     if (!spelling_pipe_in) {
 	int in, out, a = 0;
-	char *arg[10];
-	arg[a++] = "ispell";
-	arg[a++] = "-a";
-	if (option_alternate_dictionary)
-	    if (*option_alternate_dictionary) {
+	char *arg[20];
+	if (PATH_search ("aspell")) {
+	    arg[a++] = "aspell";
+	    arg[a++] = "--sug-mode=ultra";
+	    if (option_alternate_dictionary && *option_alternate_dictionary) {
 		arg[a++] = "-d";
 		arg[a++] = option_alternate_dictionary;
 	    }
-	arg[a++] = "-a";
-	arg[a++] = 0;
+	    arg[a++] = "-a";
+	    arg[a++] = 0;
+	} else {
+	    arg[a++] = "ispell";
+	    arg[a++] = "-a";
+	    if (option_alternate_dictionary && *option_alternate_dictionary) {
+		arg[a++] = "-d";
+		arg[a++] = option_alternate_dictionary;
+	    }
+	    arg[a++] = "-a";
+	    arg[a++] = 0;
+	}
 /* start ispell process */
 	ispell_pid = triple_pipe_open (&in, &out, 0, 1, arg[0], arg);
+	if (ispell_pid < 1 && errno == ENOENT) {	/* ispell not present in the path */
+	    static int tries = 0;
+	    option_auto_spellcheck = 0;
+	    if (++tries >= 2)
+		CErrorDialog (0, 0, 0, _(" Spelling Message "), "%s",
+			      _
+			      (" The ispell (or aspell) program could not be found in your path. \n Check that it is in your path and works with the -a option. "));
+	    return 1;
+	}
 	if (ispell_pid < 1) {
 	    option_auto_spellcheck = 0;
-#if 0
-	    CErrorDialog (0, 0, 0, _ (" Spelling Message "), "%s", _ (" Fail trying to open ispell program. \n Check that it is in your path and works with the -a option. \n Alternatively, disable spell checking from the Options menu. "));
-#endif
+	    CErrorDialog (0, 0, 0, _(" Spelling Message "), "%s",
+			  _
+			  (" Fail trying to open ispell (or aspell) program. \n Check that it is in your path and works with the -a option. \n Alternatively, disable spell checking from the Options menu. "));
 	    return 1;
 	}
 /* prepare pipes */
@@ -1115,7 +1136,9 @@ int edit_check_spelling (WEdit * edit)
 	spelling_pipe_out = (FILE *) fdopen (in, "w");
 	if (!spelling_pipe_in || !spelling_pipe_out) {
 	    option_auto_spellcheck = 0;
-	    CErrorDialog (0, 0, 0, _ (" Spelling Message "), "%s", _ (" Fail trying to open ispell pipes. \n Check that it is in your path and works with the -a option. \n Alternatively, disable spell checking from the Options menu. "));
+	    CErrorDialog (0, 0, 0, _(" Spelling Message "), "%s",
+			  _
+			  (" Fail trying to open ispell (or aspell) pipes. \n Check that it is in your path and works with the -a option. \n Alternatively, disable spell checking from the Options menu. "));
 	    return 1;
 	}
 /* read the banner message */
@@ -1124,7 +1147,9 @@ int edit_check_spelling (WEdit * edit)
 	    c1 = fgetc (spelling_pipe_in);
 	    if (c1 == -1 && errno != EINTR) {
 		option_auto_spellcheck = 0;
-		CErrorDialog (0, 0, 0, _ (" Spelling Message "), "%s", _ (" Fail trying to read ispell pipes. \n Check that it is in your path and works with the -a option. \n Alternatively, disable spell checking from the Options menu. "));
+		CErrorDialog (0, 0, 0, _(" Spelling Message "), "%s",
+			      _
+			      (" Fail trying to read ispell (or aspell) pipes. \n Check that it is in your path and works with the -a option. \n Alternatively, disable spell checking from the Options menu. "));
 		return 1;
 	    }
 	    if (c1 == '\n')
@@ -1133,7 +1158,8 @@ int edit_check_spelling (WEdit * edit)
     }
 /* spellcheck the word under the cursor */
     if (edit_check_spelling_at (edit, edit->curs1)) {
-	CMessageDialog (0, 0, 0, 0, _ (" Spelling Message "), "%s", _ (" Error reading from ispell. \n Ispell is being restarted. "));
+	CMessageDialog (0, 0, 0, 0, _(" Spelling Message "), "%s",
+			_(" Error reading from ispell (or aspell). \n Ispell is being restarted. "));
       close_spelling:
 	fclose (spelling_pipe_in);
 	spelling_pipe_in = 0;
@@ -1204,7 +1230,7 @@ void edit_free_syntax_rules (WEdit * edit)
     syntax_free (edit->rules);
 }
 
-#define CURRENT_SYNTAX_RULES_VERSION "68"
+#define CURRENT_SYNTAX_RULES_VERSION "72"
 
 char *syntax_text[] = {
 "# syntax rules version " CURRENT_SYNTAX_RULES_VERSION,
@@ -1239,19 +1265,22 @@ char *syntax_text[] = {
 "file ..\\*\\\\.(diff|rej|patch)$ Diff\\sOutput ^diff",
 "include diff.syntax",
 "",
-"file ..\\*\\\\.lsm$ LSM\\sFile",
+"file ..\\*\\\\.lsm$ LSM\\sFile\\sVersion\\s4 ^Begin4",
+"include lsm4.syntax",
+"",
+"file ..\\*\\\\.lsm$ LSM\\sFile\\sVersion\\s3 ^Begin3",
 "include lsm.syntax",
 "",
-"file ..\\*\\\\.sh$ Shell\\sScript ^#!\\s\\*/.\\*/(ksh|bash|sh|pdkzsh)",
+"file ..\\*\\\\.sh$ Shell\\sScript ^#!(\\s|)\\*/.\\*/(ksh|bash|sh|pdkzsh)",
 "include sh.syntax",
 "",
-"file ..\\*\\\\.(pl|PL])$ Perl\\sProgram ^#!\\s\\*/.\\*/perl",
+"file ..\\*\\\\.(pl|PL)$ Perl\\sProgram ^#!(\\s|)\\*/.\\*/perl",
 "include perl.syntax",
 "",
-"file ..\\*\\\\.(py|PY])$ Python\\sProgram ^#!\\s\\*/.\\*/python",
+"file ..\\*\\\\.(py|PY)$ Python\\sProgram ^#!(\\s|)\\*/.\\*/python",
 "include python.syntax",
 "",
-"file ..\\*\\\\.(man|[0-9n]|[0-9]x)$ NROFF\\sSource",
+"file ..\\*\\\\.(man|[0-9n]|[0-9]x)$ NROFF\\sMan\\sPage ^(.\\\"\\s|.de\\s|.ds\\s|.\"Generated\\sby\\sdb2man.xsl|...\\\"\\s$Header|.so\\sman|.TH\\s)",
 "include nroff.syntax",
 "",
 "file ..\\*\\\\.(htm|html|HTM|HTML)$ HTML\\sFile",
@@ -1323,6 +1352,9 @@ char *syntax_text[] = {
 "file ..\\*\\\\.(sql|SQL)$ SQL\\sFile",
 "include sql.syntax",
 "",
+"file ..\\*\\\\.(scm|SCM)$ Gimp\\sScript-Fu",
+"include scm.syntax",
+"",
 "file Don_t_match_me Mail\\sfolder ^From\\s",
 "include mail.syntax",
 "",
@@ -1446,8 +1478,38 @@ FILE *upgrade_syntax_file (char *syntax_file)
     return f;
 }
 
+static int apply_syntax_rules (WEdit * edit, FILE * f, int line, char *syntax_type)
+{
+    int line_error, result = 0;
+    line_error = edit_read_syntax_rules (edit, f);
+    if (line_error) {
+	if (!error_file_name)	/* an included file */
+	    result = line + line_error;
+	else
+	    result = line_error;
+    } else {
+	syntax_free (edit->syntax_type);
+	edit->syntax_type = (char *) strdup (syntax_type);
+/* if there are no rules then turn off syntax highlighting for speed */
+	if (!edit->rules[1])
+	    if (!edit->rules[0]->keyword[1] && !edit->rules[0]->spelling) {
+		edit_free_syntax_rules (edit);
+		return result;
+	    }
+/* notify the callback of a change in rule set */
+	if (syntax_change_callback)
+#ifdef MIDNIGHT
+	    (*syntax_change_callback) (&edit->widget);
+#else
+	    (*syntax_change_callback) (edit->widget);
+#endif
+    }
+    return result;
+}
+
 /* returns -1 on file error, line number on error in file syntax */
-static int edit_read_syntax_file (WEdit * edit, char **names, char *syntax_file, char *editor_file, char *first_line, char *type)
+static int edit_read_syntax_file (WEdit * edit, char **names, char *syntax_file, char *editor_file,
+				  char *first_line, char *type)
 {
     FILE *f;
     regex_t r;
@@ -1457,6 +1519,9 @@ static int edit_read_syntax_file (WEdit * edit, char **names, char *syntax_file,
     int argc;
     int result = 0;
     int count = 0;
+    int max_score = 0;
+    long offset_at_max_score = 0;
+    char *syntax_type = NULL;
     f = upgrade_syntax_file (syntax_file);
     if (!f)
 	return -1;
@@ -1485,11 +1550,14 @@ static int edit_read_syntax_file (WEdit * edit, char **names, char *syntax_file,
 	    names[count] = 0;
 	} else if (type) {
 /* 2: rule set was explicitly specified by the caller */
-	    if (!strcmp (type, args[2]))
-		goto found_type;
+	    if (!strcmp (type, args[2])) {
+		result = apply_syntax_rules (edit, f, line, args[2]);
+		break;
+	    }
 	} else if (editor_file && edit) {
 /* 3: auto-detect rule set from regular expressions */
-	    int q;
+	    int q = 0;
+	    memset (&r, '\0', sizeof (r));
 	    if (regcomp (&r, args[1], REG_EXTENDED)) {
 		result = line;
 		break;
@@ -1497,46 +1565,31 @@ static int edit_read_syntax_file (WEdit * edit, char **names, char *syntax_file,
 /* does filename match arg 1 ? */
 	    q = !regexec (&r, editor_file, 1, pmatch, 0);
 	    regfree (&r);
-	    if (!q && args[3]) {
+	    if (args[3]) {
+		memset (&r, '\0', sizeof (r));
 		if (regcomp (&r, args[3], REG_EXTENDED)) {
 		    result = line;
 		    break;
 		}
 /* does first line match arg 3 ? */
-		q = !regexec (&r, first_line, 1, pmatch, 0);
+		q += (!regexec (&r, first_line, 1, pmatch, 0));
 		regfree (&r);
 	    }
-	    if (q) {
-		int line_error;
-	      found_type:
-		line_error = edit_read_syntax_rules (edit, f);
-		if (line_error) {
-		    if (!error_file_name)	/* an included file */
-			result = line + line_error;
-		    else
-			result = line_error;
-		} else {
-		    syntax_free (edit->syntax_type);
-		    edit->syntax_type = (char *) strdup (args[2]);
-/* if there are no rules then turn off syntax highlighting for speed */
-		    if (!edit->rules[1])
-			if (!edit->rules[0]->keyword[1] && !edit->rules[0]->spelling) {
-			    edit_free_syntax_rules (edit);
-			    break;
-			}
-/* notify the callback of a change in rule set */
-		    if (syntax_change_callback)
-#ifdef MIDNIGHT
-			(*syntax_change_callback) (&edit->widget);
-#else
-			(*syntax_change_callback) (edit->widget);
-#endif
-		}
-		break;
+	    if (q > max_score) {
+		max_score = q;
+		offset_at_max_score = ftell (f);
+		syntax_type = (char *) strdup ((char *) args[2]);
 	    }
 	}
 	free_args (args);
     }
+    if (editor_file && edit)
+	if (!result && max_score) {
+	    fseek (f, offset_at_max_score, 0);
+	    result = apply_syntax_rules (edit, f, line, syntax_type);
+	}
+    if (syntax_type)
+	free (syntax_type);
     free_args (args);
     syntax_free (l);
     fclose (f);
