@@ -67,6 +67,20 @@
 #include "mad.h"
 
 
+static void xdnd_send_enter (DndClass * dnd, Window window, Window from, Atom * typelist);
+static void xdnd_send_position (DndClass * dnd, Window window, Window from, Atom action, int x, int y,
+				unsigned long etime);
+static void xdnd_send_status (DndClass * dnd, Window window, Window from, int will_accept, int want_position,
+			      int x, int y, int w, int h, Atom action);
+static void xdnd_send_leave (DndClass * dnd, Window window, Window from);
+static void xdnd_send_drop (DndClass * dnd, Window window, Window from, unsigned long etime);
+static void xdnd_send_finished (DndClass * dnd, Window window, Window from, int error);
+static int xdnd_convert_selection (DndClass * dnd, Window window, Window requester, Atom type);
+static void xdnd_selection_send (DndClass * dnd, XSelectionRequestEvent * request, unsigned char *data,
+				 int length);
+static int xdnd_get_selection (DndClass * dnd, Window from, Atom property, Window insert);
+
+
 /* just to remind us : */
 
 #if 0
@@ -561,7 +575,7 @@ static void xdnd_send_event (DndClass * dnd, Window window, XEvent * xevent)
     XSendEvent (dnd->display, window, 0, 0, xevent);
 }
 
-void xdnd_send_enter (DndClass * dnd, Window window, Window from, Atom * typelist)
+static void xdnd_send_enter (DndClass * dnd, Window window, Window from, Atom * typelist)
 {
     XEvent xevent;
     int n, i;
@@ -583,7 +597,7 @@ void xdnd_send_enter (DndClass * dnd, Window window, Window from, Atom * typelis
     xdnd_send_event (dnd, window, &xevent);
 }
 
-void xdnd_send_position (DndClass * dnd, Window window, Window from, Atom action, int x, int y, unsigned long time)
+static void xdnd_send_position (DndClass * dnd, Window window, Window from, Atom action, int x, int y, unsigned long time)
 {
     XEvent xevent;
 
@@ -605,7 +619,7 @@ void xdnd_send_position (DndClass * dnd, Window window, Window from, Atom action
     xdnd_send_event (dnd, window, &xevent);
 }
 
-void xdnd_send_status (DndClass * dnd, Window window, Window from, int will_accept, \
+static void xdnd_send_status (DndClass * dnd, Window window, Window from, int will_accept, \
 	      int want_position, int x, int y, int w, int h, Atom action)
 {
     XEvent xevent;
@@ -631,7 +645,7 @@ void xdnd_send_status (DndClass * dnd, Window window, Window from, int will_acce
     xdnd_send_event (dnd, window, &xevent);
 }
 
-void xdnd_send_leave (DndClass * dnd, Window window, Window from)
+static void xdnd_send_leave (DndClass * dnd, Window window, Window from)
 {
     XEvent xevent;
 
@@ -648,7 +662,7 @@ void xdnd_send_leave (DndClass * dnd, Window window, Window from)
     xdnd_send_event (dnd, window, &xevent);
 }
 
-void xdnd_send_drop (DndClass * dnd, Window window, Window from, unsigned long time)
+static void xdnd_send_drop (DndClass * dnd, Window window, Window from, unsigned long time)
 {
     XEvent xevent;
 
@@ -669,7 +683,7 @@ void xdnd_send_drop (DndClass * dnd, Window window, Window from, unsigned long t
 
 /* error is not actually used, i think future versions of the protocol should return an error status
    to the calling window with the XdndFinished client message */
-void xdnd_send_finished (DndClass * dnd, Window window, Window from, int error)
+static void xdnd_send_finished (DndClass * dnd, Window window, Window from, int error)
 {
     XEvent xevent;
 
@@ -687,7 +701,7 @@ void xdnd_send_finished (DndClass * dnd, Window window, Window from, int error)
 }
 
 /* returns non-zero on error - i.e. no selection owner set. Type is of course the mime type */
-int xdnd_convert_selection (DndClass * dnd, Window window, Window requester, Atom type)
+static int xdnd_convert_selection (DndClass * dnd, Window window, Window requester, Atom type)
 {
     if (!(window = XGetSelectionOwner (dnd->display, dnd->XdndSelection))) {
 	dnd_debug1 ("xdnd_convert_selection(): XGetSelectionOwner failed");
@@ -699,16 +713,16 @@ int xdnd_convert_selection (DndClass * dnd, Window window, Window requester, Ato
 }
 
 /* returns non-zero on error */
-int xdnd_set_selection_owner (DndClass * dnd, Window window, Atom type)
+static int xdnd_set_selection_owner (DndClass * dnd, Window window, Atom type, Time time)
 {
-    if (!XSetSelectionOwner (dnd->display, dnd->XdndSelection, window, CurrentTime)) {
+    if (!XSetSelectionOwner (dnd->display, dnd->XdndSelection, window, time)) {
 	dnd_debug1 ("xdnd_set_selection_owner(): XSetSelectionOwner failed");
 	return 1;
     }
     return 0;
 }
 
-void xdnd_selection_send (DndClass * dnd, XSelectionRequestEvent * request, unsigned char *data, int length)
+static void xdnd_selection_send (DndClass * dnd, XSelectionRequestEvent * request, unsigned char *data, int length)
 {
     XEvent xevent;
     dnd_debug2 ("      requestor = %ld", request->requestor);
@@ -787,7 +801,7 @@ static int paste_prop_internal (DndClass * dnd, Window from, Window insert, unsi
 /*
  * Respond to a notification that a primary selection has been sent (supports INCR)
  */
-int xdnd_get_selection (DndClass * dnd, Window from, Atom prop, Window insert)
+static int xdnd_get_selection (DndClass * dnd, Window from, Atom prop, Window insert)
 {
     struct timeval tv, tv_start;
     long nread;
@@ -1190,7 +1204,7 @@ Atom xdnd_drag (DndClass * dnd, Window from, Atom action, Atom * typelist)
 		}
 	    }
 	} else {
-	    xdnd_set_selection_owner (dnd, from, dnd->desired_type);
+	    xdnd_set_selection_owner (dnd, from, dnd->desired_type, time);
 #if XDND_VERSION < 3
 	    xdnd_send_drop (dnd, dnd->dropper_window, from, time);
 #else
